#!/bin/sh

RETVAL=1
action=$1

basename=${0##*/}
dirname=${0%$basename}
dirname=${dirname:-.}
dirname=`cd $dirname; echo $PWD`

usage () {
  echo "usage: $basename {setup|cleanup|cleanup-nocheck|dumpconfig|kinit}"
  exit 1
}

test "$#" -ne 1 && { usage; exit 1; }

check () {
  echo " * Checking packages:"
  checkpkgs="
    krb5-server
    krb5-workstation
  "
  for pkg in $checkpkgs; do
    printf "  - "
    rpm -q $pkg || return 1
  done
}

cleanup () {
  echo " * Cleaning up:"
  pkill qpidd && echo "   - qpidd killed"
  /etc/init.d/krb5kdc stop >/dev/null 2>&1 && echo "   - krb5kdc stopped"
  /etc/init.d/kadmin stop >/dev/null 2>&1 && echo "   - kadmin stopped"
  rm -rf /etc/krb5.conf /etc/krb5.keytab /var/kerberos/krb5kdc/{.k*,*} \
    && echo "   - all config files deleted"
  kdestroy && echo " * Tickets destroyed"
}

setup () {
  echo " * Setting up Kerberos5..."
  . $dirname/krb5.conf
  . $dirname/krb5-files
  restorecon -R /etc /var

  echo "   - Creating database..." &&
    echo -e "$DPASSW\n$DPASSW" | kdb5_util create -s \
    >& /dev/null

  echo -e "$APASSW\n$APASSW" | kadmin.local -q 'addprinc admin/admin' \
    >& /dev/null

  echo "   - Starting krb5kdc and kadmin..." &&
    /etc/init.d/krb5kdc start >& /dev/null &&
    /etc/init.d/kadmin start >& /dev/null

  echo "   - Getting ticket for admin account... " &&
    echo "$APASSW" |
    kinit admin/admin \
    >& /dev/null

  echo "   - Adding qpidd/$FQDN principal... " &&
    echo "$APASSW" |
    kadmin -q "addprinc -randkey qpidd/$FQDN" \
    >& /dev/null &&
    echo "$APASSW" |
    kadmin -q "ktadd qpidd/$FQDN" \
    >& /dev/null

  echo "   - Adding $USERNAME principal... " &&
    echo -e "$APASSW\n$TPASSW\n$TPASSW" |
    kadmin -q "addprinc $USERNAME" \
    >& /dev/null
}

krb5_kinit () {
  . $dirname/krb5.conf
  expect -c "
    spawn kinit $USERNAME@$REALM
    expect :
    send $TPASSW\r
    expect eof { exit 0 }
    exit 1
  "
}

header () {
  echo " *-KRB5-$action"
}

footer () {
  echo " *-KRB5-$action finished"
}

set -e
case "$action" in
  s*)
    header
    check || exit 1
    cleanup
    setup && export RETVAL=0
    footer
    ;;
  cl*)
    header
    check || exit 1
    cleanup && RETVAL=0
    footer
    ;;
  cleanup-no*)
    header
    cleanup && RETVAL=0
    footer
    ;;
  dump*)
    test -z "$HOSTNAME" && HOSTNAME=`hostname -f`

    echo "FQDN=$HOSTNAME"
    grep -v HOSTNAME $dirname/krb5.conf && RETVAL=0
    ;;
  kinit)
    krb5_kinit && RETVAL=0
    ;;
  *)
    usage
esac

exit $RETVAL
